
EPROM_START    equ $F000
RAM_START      equ $4000
STACK_START    equ $7FFF
RESET_VEC      equ $FFFE
FAIL_ADDR      equ $FF00

; Start of EPROM
            org  EPROM_START
start:      movb #8,$16
            movb #4,$0a
            ldx  #RAM_START
            ;ldd  #$55AA
            ;bra  mem_test
            ;ldd  #$AA55
            ;bra  mem_test
            lds  #STACK_START
            bra  run_prog
            
; Rewritten version of Lab 2
;mem_test:
;mt_loop:
;            movw #$0,Y,0,X
;            ldd  2,X+
;            cmpd $Y
;            lbne FAIL_ADDR
;            cpx  STACK_START
;            blt  mt_loop
            
run_prog:   movw #$D70A,1,-SP
            movw #$3CA3,2,-SP
            movw #$D70A,2,-SP
            movw #$3BA3,2,-SP
            bsr  add_floats
            
while_1:    bra while_1       ; loop forever
                        
; Adds the 24-bit number at X with the 24-bit number at X + 3.
add_24_unsigned: ; *(X) = *(X) + *(X + 3)
            leax  2,X
            ldaa  0,X
            adda  3,X
            staa  1,X-
            ldaa  0,X
            adca  3,X
            staa  1,X-
            ldaa  0,X
            adca  3,X
            staa  1,X-
            rts

; Adds the 2 24-bit numbers pushed on the stack and stores the sum in the last push.
add_24_stack:
            ldd  3,SP
            addd 6,SP
            std  3,SP
            ldaa 2,SP
            adca 5,SP
            staa 2,SP
            rts

; Gets the 2's complement of the 24-bit number (big endian) starting at X.
neg_24_unsigned:
            com  1,X+
            com  1,X+
            com  0,X
            ldd  1,-X
            addd #1
            std  0,X
            bne  n24u_done
            inc  1,-X
n24u_done:  rts
            

; Gets the 2's complement of the 24-bit number (big endian) pushed on the stack.
neg_24_stack:
            com  2,SP
            com  3,SP
            com  4,SP
            ldd  3,SP
            addd #1
            std  3,SP
            bne  n24s_done
            inc  2,SP
n24s_done:  rts

; Subtracts the 24-bit number pushed last from the 24-bit number pushed first.
sub_24_stack:
            bra  neg_24_stack
            bra  add_24_stack
            rts

; shiftR_24_stack - Shifts a 24-bit number <num> to the right <N> places.
; @param    num   @loc[SP+3]: the number to shift right.
; @param    N     @loc[SP+2]: the number of bits to shift
; @return   num   @loc[SP+3]: New number overwrites the old one.
shiftR_24_stack:
            ldaa 5,SP      ; A = N (bits to shift)
            cmpa #24       ; if (N >= 24)
            bge  s24s_end  ; return
s24s_if0:   tsta           ; if (N == 0)
            beq  s24s_end  ; return
            cmpa #8        ; if (N < 8)
            blt  s24s_1    ; shift right 1 and continue
            cmpa #16       ; if (N < 16)
            blt  s24s_8    ; shift right 8 and continue
s24s_16:    movb 2,SP,4,SP ; num[7:0] = num[23:16]
            clr  2,SP      ; num[23:16] = 0
            clr  3,SP      ; num[15:8] = 0
            suba #16       ; N = N - 16
            bra  s24s_if0  ; continue
s24s_8:     movw 2,SP,3,SP ; num[15:0] = num[23:8]
            clr  2,SP      ; num[23:16] = 0
            suba #8        ; N = N - 8
            bra  s24s_if0  ; continue
s24s_1:     lsr  2,SP      ; num[23:16] = {0, num[23:17]}
            ror  3,SP      ; num[15:8] = num[16:9]
            ror  4,SP      ; num[7:0] = num[8:1]
            deca           ; N--
            bra  s24s_if0  ; continue
s24s_end:   rts

; Add 2 32-bit floats.
add_floats:
            leax 6,SP
            leas -10,SP
            ldd  0,X
            andb #$80
            lsld
            rolb
            std  8,SP
            ldd  -4,X
            andb #$80
            lsld
            rolb
            std  6,SP
            suba 8,SP
            tsta
            bgt  norm_flt1
norm_flt2:  nega
            staa 3,SP
            movb -1,X,2,SP
            ldd  -3,X
            anda #$7F
            std  0,SP
            bsr  shiftR_24_stack
            tst  7,SP
            bne  neg_flt2
get_flt1:   movb 3,X,5,SP
            ldd  1,X
            anda #$7F
            std  3,SP
            jsr  add_24_stack
            jmp  while_1
neg_flt2:   jsr  neg_24_stack
            bra  get_flt1
norm_flt1:  staa 3,SP
            movb 3,X,2,SP
            ldd  1,X
            anda #$7F
            std  0,SP
            jsr  shiftR_24_stack
            tst  9,SP
            bne  neg_flt1
get_flt2:   movb -1,X,5,SP
            ldd  -3,X
            anda #$7F
            std  3,SP
            jsr  add_24_stack
            jmp  while_1
neg_flt1:   jsr  neg_24_stack
            bra  get_flt2
            

            


         org RESET_VEC
         dw  EPROM_START
